/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework.message;

import java.net.URI;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.security.auth.Subject;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.ow2.petals.component.framework.Constants.MEPConstants;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.SourceHelper;
import org.w3c.dom.Document;

/**
 * This is a Message Exchange Wrapper. The "in" and "out" normalized message are
 * cached into this class, authorizing the reset of these message. Can be use by
 * the Component Framework interceptors.
 * 
 * @author ofabre - EBM WebSourcing
 * @since 3.0
 * 
 */
public class ExchangeImpl implements Exchange {

    private MessageExchange exchange;

    private NormalizedMessage inNormalizedExchange;

    private NormalizedMessage outNormalizedExchange;

    private boolean checkRole;

    /**
     * 
     * Creates a new instance of {@link ExchangeImpl}
     * 
     * @param exchange
     */
    public ExchangeImpl(MessageExchange exchange) {
        super();
        this.exchange = exchange;
        this.inNormalizedExchange = this.exchange.getMessage(IN_MESSAGE_NAME);
        this.outNormalizedExchange = this.exchange.getMessage(OUT_MESSAGE_NAME);
        this.checkRole = true;
    }

    /**
     * Creates a message
     * 
     * @return
     * @throws MessagingException
     */
    public NormalizedMessage createMessage() throws MessagingException {
        return this.exchange.createMessage();
    }

    /**
     * Return the Wrapper Message Exchange. Set the "in" or the "out" normalized
     * messages if they are not already set.
     *
     * @return
     */
    public MessageExchange getMessageExchange() {
        try {
            if ((this.inNormalizedExchange != null)
                    && (this.exchange.getMessage(IN_MESSAGE_NAME) == null)) {
                this.exchange.setMessage(this.inNormalizedExchange, IN_MESSAGE_NAME);
            }

            if ((this.outNormalizedExchange != null)
                    && (this.exchange.getMessage(OUT_MESSAGE_NAME) == null)) {
                this.exchange.setMessage(this.outNormalizedExchange, OUT_MESSAGE_NAME);
            }
        } catch (MessagingException e) {
            // do nothing
        }

        return this.exchange;
    }

    /**
     * Implements {@link Exchange#createFault()}
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#createFault()
     */
    public Fault createFault() throws MessagingException {
        return this.exchange.createFault();
    }

    /**
     * Implements {@link Exchange#getEndpoint()}
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getEndpoint()
     */
    public ServiceEndpoint getEndpoint() {
        return this.exchange.getEndpoint();
    }

    /**
     * Implements {@link Exchange#getError()}
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getError()
     */
    public Exception getError() {
        return this.exchange.getError();
    }

    /**
     * Implements {@link Exchange#getExchangeId()}
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getExchangeId()
     */
    public String getExchangeId() {
        return this.exchange.getExchangeId();
    }

    /**
     * Implements {@link Exchange#getFault()}
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getFault()
     */
    public Fault getFault() {
        return this.exchange.getFault();
    }

    /**
     * Implements {@link Exchange#getInterfaceName()}
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getInterfaceName()
     */
    public QName getInterfaceName() {
        return this.exchange.getInterfaceName();
    }

    /**
     * Implements {@link Exchange#getOperation()}
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getOperation()
     */
    public QName getOperation() {
        return this.exchange.getOperation();
    }

    /**
     * Implements {@link Exchange#getPattern()}
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getPattern()
     */
    public URI getPattern() {
        return this.exchange.getPattern();
    }

    /**
     * Implements {@link Exchange#getProperty(String)}
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getProperty(String)
     */
    public Object getProperty(final String name) {
        return this.exchange.getProperty(name);
    }

    /**
     * Implements {@link Exchange#getPropertyNames()}
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getPropertyNames()
     */
    public Set<?> getPropertyNames() {
        return this.exchange.getPropertyNames();
    }

    /**
     * Implements {@link Exchange#getRole()}
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getRole()
     */
    public MessageExchange.Role getRole() {
        return this.exchange.getRole();
    }

    /**
     * Implements {@link Exchange#getService()}
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getService()
     */
    public QName getService() {
        return this.exchange.getService();
    }

    /**
     * Implements {@link Exchange#getStatus()}
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#getStatus()
     */
    public ExchangeStatus getStatus() {
        return this.exchange.getStatus();
    }

    /**
     * Implements {@link Exchange#isTransacted()}
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#isTransacted()
     */
    public boolean isTransacted() {
        return this.exchange.isTransacted();
    }

    /**
     * Implements {@link Exchange#setEndpoint(ServiceEndpoint)}
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#setEndpoint(ServiceEndpoint)
     */
    public void setEndpoint(final ServiceEndpoint endpoint) {
        this.exchange.setEndpoint(endpoint);
    }

    /**
     * Implements {@link Exchange#setError(Exception)}
     * 
     * Put the given exception as Error part of the message exchange. Allowed
     * exception types are : {@link java.*}, {@link javax.jbi.JBIException},
     * org.w3c.* and org.xml.*.
     * 
     * @see org.ow2.petals.component.framework.api.message.Exchange#setError(Exception)
     */
    public void setError(final Exception error) {
        String packageName = error.getClass().getPackage().getName();
        if (packageName.startsWith("java") || packageName.startsWith("org.w3c")
                || packageName.startsWith("org.xml")) {
            this.exchange.setError(error);
        } else {
            throw new IllegalArgumentException("Exception of type (" + packageName
                    + ") are not supported by the JBI container");
        }
    }

    public void setFault(Fault fault) throws MessagingException {
        this.exchange.setFault(fault);
    }

    public void setFault(Exception e) throws MessagingException {
        final Fault fault = this.exchange.createFault();
        final Document faultDocument = SourceHelper.createSoapFaultDocument(e,
                ExchangeImpl.DEFAULT_FAULT_QNAME);

        if (SourceHelper.isDOM3(faultDocument)) {
            fault.setContent(SourceHelper.createSource(faultDocument));
        } else {
            Source inSource;
            try {
                inSource = SourceHelper.convertDOM2ToSource(faultDocument);
            } catch (PEtALSCDKException e1) {
                throw new MessagingException(e1);
            }
            fault.setContent(inSource);
        }

        this.exchange.setFault(fault);
    }

    public void setInterfaceName(QName interfaceName) {
        this.exchange.setInterfaceName(interfaceName);
    }

    public void setOperation(QName name) {
        this.exchange.setOperation(name);
    }

    public void setProperty(String name, Object obj) {
        this.exchange.setProperty(name, obj);
    }

    public void setService(QName service) {
        this.exchange.setService(service);
    }

    public void setStatus(ExchangeStatus status) throws MessagingException {
        this.exchange.setStatus(status);
    }

    /**
     * @deprecated use the method ${@link #setOutMessageContent(Document))
     */
    public void setOutMessageContent(String outContent) throws MessagingException {
        try {
            NormalizedMessage outMess = this.getOutMessage();
            if (outMess == null) {
                outMess = this.exchange.createMessage();
            }
            outMess.setContent(SourceHelper.createSourceFromString(outContent));
            this.setOutMessage(outMess);
        } catch (PEtALSCDKException e) {
            throw new MessagingException("Error creating response message");
        }
    }

    public void setOutMessageContent(Document outContent) throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }

        if (SourceHelper.isDOM3(outContent)) {
            outMess.setContent(SourceHelper.createSource(outContent));
        } else {
            Source inSource;
            try {
                inSource = SourceHelper.convertDOM2ToSource(outContent);
            } catch (PEtALSCDKException e) {
                throw new MessagingException(e);
            }
            outMess.setContent(inSource);
        }

        this.setOutMessage(outMess);
    }

    public void setOutMessageContent(Source outContent) throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }
        outMess.setContent(outContent);
        this.setOutMessage(outMess);
    }

    /**
     * @deprecated use the method ${@link #setInMessageContent(Document))
     */
    public void setInMessageContent(String inContent) throws MessagingException {
        try {
            NormalizedMessage inMess = this.getInMessage();
            if (inMess == null) {
                inMess = this.exchange.createMessage();
            }
            inMess.setContent(SourceHelper.createSourceFromString(inContent));
            this.setInMessage(inMess);
        } catch (PEtALSCDKException e) {
            throw new MessagingException("Error creating request message");
        }
    }

    public void setInMessageContent(Document inContent) throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }

        if (SourceHelper.isDOM3(inContent)) {
            inMess.setContent(SourceHelper.createSource(inContent));
        } else {
            Source inSource;
            try {
                inSource = SourceHelper.convertDOM2ToSource(inContent);
            } catch (PEtALSCDKException e) {
                throw new MessagingException(e);
            }
            inMess.setContent(inSource);
        }

        this.setInMessage(inMess);
    }

    public void setInMessageContent(Source inContent) throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }
        inMess.setContent(inContent);
        this.setInMessage(inMess);
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.api.message.Exchange#setOutMessageAttachments(java.util.Map)
     */
    public void setOutMessageAttachments(Map<String, DataHandler> attachments)
            throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }
        for (String attachementId : attachments.keySet()) {
            outMess.addAttachment(attachementId, attachments.get(attachementId));
        }
        this.setOutMessage(outMess);
    }
    
    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.api.message.Exchange#setInMessageAttachments(java.util.Map)
     */
    public void setInMessageAttachments(Map<String, DataHandler> attachments)
            throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }
        for (String attachementId : attachments.keySet()) {
            inMess.addAttachment(attachementId, attachments.get(attachementId));
        }
        this.setInMessage(inMess);
    }


    /**
     * @deprecated use the method ${@link #getInMessageContentAsDocument()
     */
    @Deprecated
    public String getInMessageContentAsString() throws MessagingException {
        NormalizedMessage inMessage = this.getInMessage();
        if (inMessage == null) {
            throw new MessagingException("In message must not be null");
        }
        Source content = inMessage.getContent();
        String stringContent = null;
        try {
            stringContent = SourceHelper.convertToString(content);
        } catch (PEtALSCDKException e) {
            throw new MessagingException("Error creating string from source", e);
        }
        return stringContent;
    }

    public Document getInMessageContentAsDocument() throws MessagingException {
        final NormalizedMessage inMessage = this.getInMessage();
        if (inMessage == null) {
            throw new MessagingException("In message must not be null");
        }
        final Source content = inMessage.getContent();
        Document documentContent = null;
        try {
            documentContent = SourceHelper.createDocument(content);
        } catch (PEtALSCDKException e) {
            throw new MessagingException("Error creating string from source", e);
        }
        return documentContent;
    }

    public Source getInMessageContentAsSource() throws MessagingException {
        NormalizedMessage inMessage = this.getInMessage();
        if (inMessage == null) {
            throw new MessagingException("In message must not be null");
        }
        return inMessage.getContent();
    }

    /**
     * @deprecated use the method ${@link #getOutMessageContentAsDocument()
     */
    @Deprecated
    public String getOutMessageContentAsString() throws MessagingException {
        NormalizedMessage outMessage = this.getOutMessage();
        if (outMessage == null) {
            throw new MessagingException("Out message must not be null");
        }
        Source content = outMessage.getContent();
        String stringContent = null;
        try {
            stringContent = SourceHelper.convertToString(content);
        } catch (PEtALSCDKException e) {
            throw new MessagingException("Error creating string from source", e);
        }
        return stringContent;
    }

    public Document getOutMessageContentAsDocument() throws MessagingException {
        final NormalizedMessage outMessage = this.getOutMessage();
        if (outMessage == null) {
            throw new MessagingException("Out message must not be null");
        }
        final Source content = outMessage.getContent();
        Document documentContent = null;
        try {
            documentContent = SourceHelper.createDocument(content);
        } catch (PEtALSCDKException e) {
            throw new MessagingException("Error creating string from source", e);
        }
        return documentContent;
    }

    public Source getOutMessageContentAsSource() throws MessagingException {
        NormalizedMessage outMessage = this.getOutMessage();
        if (outMessage == null) {
            throw new MessagingException("Out message must not be null");
        }
        return outMessage.getContent();
    }

    public Set<DataHandler> getInMessageAttachments() throws MessagingException {
        Set<DataHandler> attachments = new HashSet<DataHandler>();

        NormalizedMessage inMessage = this.getInMessage();
        if (inMessage == null) {
            throw new MessagingException("In message must not be null");
        }

        Set<?> names = inMessage.getAttachmentNames();
        for (Object attachId : names) {
            DataHandler dh = inMessage.getAttachment((String) attachId);
            if (dh != null)
                attachments.add(dh);
        }
        return attachments;
    }

    public URI getExchangePattern() throws MessagingException {
        URI pattern = this.exchange.getPattern();
        if (pattern == null) {
            throw new MessagingException(
                    "You must specify a message exchange pattern in the message exchange");
        }
        return pattern;
    }

    public String getOperationName() throws MessagingException {
        QName operation = this.exchange.getOperation();
        if (operation == null) {
            throw new MessagingException("You must specify an operation the message exchange");
        }
        return operation.getLocalPart();
    }

    public String getEndpointName() throws MessagingException {
        ServiceEndpoint ep = this.exchange.getEndpoint();
        if (ep == null) {
            throw new MessagingException("You must specify an endpoint in the message exchange");
        }
        return ep.getEndpointName();
    }

    public NormalizedMessage getInMessage() {
        if (this.inNormalizedExchange == null) {
            if (this.exchange.getMessage(IN_MESSAGE_NAME) != null) {
                return this.exchange.getMessage(IN_MESSAGE_NAME);
            }
            try {
                this.inNormalizedExchange = this.exchange.createMessage();
            } catch (MessagingException e) {
                return null;
            }
        }
        return this.inNormalizedExchange;
    }

    public boolean isInMessage() {
        return this.exchange.getMessage(IN_MESSAGE_NAME) != null
                || this.inNormalizedExchange != null;
    }

    public void setInMessage(final NormalizedMessage msg) throws MessagingException {
        this.checkPatternMatching(IN_MESSAGE_NAME);
        this.checkRoleMatching(IN_MESSAGE_NAME);
        this.inNormalizedExchange = msg;
    }

    public NormalizedMessage getOutMessage() {
        if (this.outNormalizedExchange == null) {
            if (this.exchange.getMessage(OUT_MESSAGE_NAME) != null) {
                return this.exchange.getMessage(OUT_MESSAGE_NAME);
            } else {
                try {
                    this.outNormalizedExchange = this.exchange.createMessage();
                } catch (MessagingException e) {
                    return null;
                }
            }
        }
        return this.outNormalizedExchange;
    }

    public boolean isOutMessage() {
        return this.exchange.getMessage(OUT_MESSAGE_NAME) != null
                || this.outNormalizedExchange != null;
    }

    public void setOutMessage(final NormalizedMessage msg) throws MessagingException {
        this.checkPatternMatching(OUT_MESSAGE_NAME);
        this.checkRoleMatching(OUT_MESSAGE_NAME);
        this.outNormalizedExchange = msg;
    }

    /* ### STATUS ### */

    public void setDoneStatus() throws MessagingException {
        this.setStatus(ExchangeStatus.DONE);
    }

    public boolean isDoneStatus() {
        return (ExchangeStatus.DONE.equals(this.getStatus()));
    }

    public void setErrorStatus() throws MessagingException {
        this.setStatus(ExchangeStatus.ERROR);
    }

    public boolean isErrorStatus() {
        return (ExchangeStatus.ERROR.equals(this.getStatus()));
    }

    public void setActiveStatus() throws MessagingException {
        this.setStatus(ExchangeStatus.ACTIVE);
    }

    public boolean isActiveStatus() {
        return (ExchangeStatus.ACTIVE.equals(this.getStatus()));
    }

    /* ### ROLE ### */

    public boolean isProviderRole() {
        return (MessageExchange.Role.PROVIDER.equals(this.getRole()));
    }

    public boolean isConsumerRole() {
        return (MessageExchange.Role.CONSUMER.equals(this.getRole()));
    }

    /* ### PATTERN ### */

    public boolean isInOnlyPattern() {
    	return MEPConstants.IN_ONLY_PATTERN.equals(this.exchange.getPattern());
    }

    public boolean isRobustInOnlyPattern() {
    	return MEPConstants.ROBUST_IN_ONLY_PATTERN.equals(this.exchange.getPattern());
    }

    public boolean isInOutPattern() {
    	return MEPConstants.IN_OUT_PATTERN.equals(this.exchange.getPattern());
    }

    public boolean isInOptionalOutPattern() {
    	return MEPConstants.IN_OPTIONAL_OUT_PATTERN.equals(this.exchange.getPattern());
    }

    public void setCheckRole(boolean checkRole) {
        this.checkRole = checkRole;
    }

    /**
     * Check that the exchange MEP allows the specified reference. This only
     * works for the JBI predefined MEPs. It check that reference is IN or OUT.
     * Other reference or MEP will cause an exception
     * 
     * @param name
     * @throws MessagingException
     *             the MEP does not allow the reference
     */
    protected void checkPatternMatching(String name) throws MessagingException {
        if (name == null) {
            throw new MessagingException("The reference name is not defined.");
        }

        if (OUT_MESSAGE_NAME.equalsIgnoreCase(name)
                && ((MEPConstants.IN_ONLY_PATTERN.equals(this.exchange
						.getPattern())) || MEPConstants.ROBUST_IN_ONLY_PATTERN
						.equals(this.exchange.getPattern()))) {
            throw new MessagingException("the MessageExchange state does not allow this operation.");
        }
    }

    /**
     * Check that the Role allows the specified reference. This only works for
     * the JBI predefined MEPs. It check that reference is IN or OUT and the
     * Role is CONSUMER or PROVIDER. Other reference will cause an exception
     * 
     * @param name
     * @throws MessagingException
     *             the Role does not allow the reference
     */
    protected void checkRoleMatching(String name) throws MessagingException {
        if (name == null) {
            throw new MessagingException("The reference name is not defined.");
        }

        // The check on the role can be unactivated for CDK intercepting purpose
        if (!this.checkRole) {
            return;
        }

        if (MessageExchange.Role.CONSUMER.equals(this.exchange.getRole())) {
            if (IN_MESSAGE_NAME.equalsIgnoreCase(name)) {
                return;
            }
        } else if (MessageExchange.Role.PROVIDER.equals(this.exchange.getRole())
                && OUT_MESSAGE_NAME.equalsIgnoreCase(name)) {
            return;
        }
        throw new MessagingException("The Role does not allow this operation.");
    }

    public void setOutMessageContent(String outContent, Subject securitySubject)
            throws MessagingException {
        setOutMessageContent(outContent);
        NormalizedMessage normalizedMessage = getOutMessage();
        normalizedMessage.setSecuritySubject(securitySubject);
    }

    public void setOutMessageContent(Document outContent, Subject securitySubject)
            throws MessagingException {
        setOutMessageContent(outContent);
        NormalizedMessage normalizedMessage = getOutMessage();
        normalizedMessage.setSecuritySubject(securitySubject);
    }

    public void setOutMessageContent(Source outContent, Subject securitySubject)
            throws MessagingException {
        setOutMessageContent(outContent);
        NormalizedMessage normalizedMessage = getOutMessage();
        normalizedMessage.setSecuritySubject(securitySubject);
    }
    
    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.api.message.Exchange#getInMessageProperty(java.lang.String)
     */
    public Object getInMessageProperty(String name) throws MessagingException {
        NormalizedMessage nm = this.getInMessage();
        if (nm == null) {
            throw new MessagingException("In message must not be null");
        }
        return nm.getProperty(name);
    }
    
    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.api.message.Exchange#getInMessagePropertyNames()
     */
    public Set<?> getInMessagePropertyNames() throws MessagingException {
        NormalizedMessage nm = this.getInMessage();
        if (nm == null) {
            throw new MessagingException("In message must not be null");
        }
        return nm.getPropertyNames();
    }
    
    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.api.message.Exchange#getOutMessageProperty(java.lang.String)
     */
    public Object getOutMessageProperty(String name) throws MessagingException {
        NormalizedMessage nm = this.getOutMessage();
        if (nm == null) {
            throw new MessagingException("Out message must not be null");
        }
        return nm.getProperty(name);
    }
    
    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.api.message.Exchange#getOutMessagePropertyNames()
     */
    public Set<?> getOutMessagePropertyNames() throws MessagingException {
        NormalizedMessage nm = this.getOutMessage();
        if (nm == null) {
            throw new MessagingException("Out message must not be null");
        }
        return nm.getPropertyNames();
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.api.message.Exchange#setInMessageProperty(java.lang.String, java.lang.Object)
     */
    public void setInMessageProperty(String name, Object value) throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }
        inMess.setProperty(name, value);
        this.setInMessage(inMess);
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.api.message.Exchange#setInMessageProperties(java.util.Map)
     */
    public void setInMessageProperties(Map<String, Object> properties) throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }
        Iterator<String> iter = properties.keySet().iterator();
        while (iter.hasNext()) {
            String key = iter.next();
            inMess.setProperty(key, properties.get(key));
        }
        this.setInMessage(inMess);
    }
    
    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.api.message.Exchange#setOutMessageProperties(java.util.Map)
     */
    public void setOutMessageProperties(Map<String, Object> properties) throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }
        Iterator<String> iter = properties.keySet().iterator();
        while (iter.hasNext()) {
            String key = iter.next();
            outMess.setProperty(key, properties.get(key));
        }
        this.setOutMessage(outMess);
    }
    
    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.api.message.Exchange#setOutMessageProperty(java.lang.String, java.lang.Object)
     */
    public void setOutMessageProperty(String name, Object value) throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }
        outMess.setProperty(name, value);
        this.setOutMessage(outMess);
    }
    
    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.api.message.Exchange#setOutMessageAttachment(java.lang.String, javax.activation.DataHandler)
     */
    public void addOutMessageAttachment(String id, DataHandler content)
            throws MessagingException {
        NormalizedMessage outMess = this.getOutMessage();
        if (outMess == null) {
            outMess = this.exchange.createMessage();
        }
        outMess.addAttachment(id, content);
        this.setOutMessage(outMess);
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.component.framework.api.message.Exchange#setInMessageAttachment(java.lang.String, javax.activation.DataHandler)
     */
    public void addInMessageAttachment(String id, DataHandler content)
            throws MessagingException {
        NormalizedMessage inMess = this.getInMessage();
        if (inMess == null) {
            inMess = this.exchange.createMessage();
        }
        inMess.addAttachment(id, content);
        this.setInMessage(inMess);
    }
}
